@@ -1,42 +1,36 @@
-package Email::Simple::Header;
-
 use strict;
+use warnings;
+package Email::Simple::Header;
+# ABSTRACT: the header of an Email::Simple message
+$Email::Simple::Header::VERSION = '2.208';
 use Carp ();
 
 require Email::Simple;
 
-$Email::Simple::Header::VERSION = '2.102';
-
-=head1 NAME
-
-Email::Simple::Header - the header of an Email::Simple message
-
-=head1 SYNOPSIS
-
-  my $email = Email::Simple->new($text);
-
-  my $header = $email->head;
-  print $header->as_string;
-
-=head1 DESCRIPTION
-
-This method implements the headers of an Email::Simple object.  It is a very
-minimal interface, and is mostly for private consumption at the moment.
-
-=head1 METHODS
-
-=head2 new
-
-  my $header = Email::Simple::Header->new($head, \%arg);
-
-C<$head> is a string containing a valid email header, or a reference to such a
-string.  If a reference is passed in, don't expect that it won't be altered.
-
-Valid arguments are:
-
-  crlf - the header's newline; defaults to CRLF
-
-=cut
+#pod =head1 SYNOPSIS
+#pod
+#pod   my $email = Email::Simple->new($text);
+#pod
+#pod   my $header = $email->header_obj;
+#pod   print $header->as_string;
+#pod
+#pod =head1 DESCRIPTION
+#pod
+#pod This method implements the headers of an Email::Simple object.  It is a very
+#pod minimal interface, and is mostly for private consumption at the moment.
+#pod
+#pod =method new
+#pod
+#pod   my $header = Email::Simple::Header->new($head, \%arg);
+#pod
+#pod C<$head> is a string containing a valid email header, or a reference to such a
+#pod string.  If a reference is passed in, don't expect that it won't be altered.
+#pod
+#pod Valid arguments are:
+#pod
+#pod   crlf - the header's newline; defaults to CRLF
+#pod
+#pod =cut
 
 # We need to be able to:
 #   * get all values by lc name
@@ -71,27 +65,30 @@ sub _header_to_list {
 
   while ($$head =~ m/\G(.+?)$crlf/go) {
     local $_ = $1;
-    if (s/^\s+// or not /^([^:]+):\s*(.*)/) {
+
+    if (/^\s+/ or not /^([^:]+):\s*(.*)/) {
       # This is a continuation line. We fold it onto the end of
       # the previous header.
       next if !@headers;  # Well, that sucks.  We're continuing nothing?
 
-      $headers[-1] .= $headers[-1] ? " $_" : $_;
+      (my $trimmed = $_) =~ s/^\s+//;
+      $headers[-1][0] .= $headers[-1][0] =~ /\S/ ? " $trimmed" : $trimmed;
+      $headers[-1][1] .= "$mycrlf$_";
     } else {
-      push @headers, $1, $2;
+      push @headers, $1, [ $2, $_ ];
     }
   }
 
   return \@headers;
 }
 
-=head2 as_string
-
-  my $string = $header->as_string(\%arg);
-
-This returns a stringified version of the header.
-
-=cut
+#pod =method as_string
+#pod
+#pod   my $string = $header->as_string(\%arg);
+#pod
+#pod This returns a stringified version of the header.
+#pod
+#pod =cut
 
 # RFC 2822, 3.6:
 # ...for the purposes of this standard, header fields SHOULD NOT be reordered
@@ -115,22 +112,24 @@ sub as_string {
   };
 
   for (my $i = 0; $i < @$headers; $i += 2) {
-    my $header = "$headers->[$i]: $headers->[$i + 1]";
+    if (ref $headers->[ $i + 1 ]) {
+      $header_str .= $headers->[ $i + 1 ][1] . $self->crlf;
+    } else {
+      my $header = "$headers->[$i]: $headers->[$i + 1]";
 
-    $header_str .= lc $headers->[$i] eq 'content-type'
-                 ? $header . $self->crlf
-                 : $self->_fold($header, $fold_arg);
+      $header_str .= $self->_fold($header, $fold_arg);
+    }
   }
 
   return $header_str;
 }
 
-=head2 header_names
-
-This method returns the unique header names found in this header, in no
-particular order.
-
-=cut
+#pod =method header_names
+#pod
+#pod This method returns a list of the unique header names found in this header, in
+#pod no particular order.
+#pod
+#pod =cut
 
 sub header_names {
   my $headers = $_[0]->{headers};
@@ -140,55 +139,93 @@ sub header_names {
     map { $headers->[ $_ * 2 ] } 0 .. int($#$headers / 2);
 }
 
-=head2 header_pairs
+#pod =method header_raw_pairs
+#pod
+#pod   my @pairs = $header->header_raw_pairs;
+#pod   my $first_name  = $pairs[0];
+#pod   my $first_value = $pairs[1];
+#pod
+#pod This method returns a list of all the field/value pairs in the header, in the
+#pod order that they appear in the header.  (Remember: don't try assigning that to a
+#pod hash.  Some fields may appear more than once!)
+#pod
+#pod =method header_pairs
+#pod
+#pod L<header_pairs> is another name for L<header_raw_pairs>, which was the original
+#pod name for the method and which you'll see most often.  In general, though, it's
+#pod better to be explicit and use L<header_raw_pairs>.  (In Email::MIME,
+#pod L<header_str_pairs> exists for letting the library do the header decoding for
+#pod you.)
+#pod
+#pod =cut
 
-This method returns all the field/value pairs in the header, in the order that
-they appear in the header.
+sub header_raw_pairs {
+  my ($self) = @_;
 
-=cut
+  my @pairs = map {; _str_value($_) } @{ $self->{headers} };
+
+  return @pairs;
+}
 
 sub header_pairs {
   my ($self) = @_;
-
-  return @{ $self->{headers} };
+  $self->header_raw_pairs;
 }
 
-=head2 header
+#pod =method header_raw
+#pod
+#pod   my $first_value = $header->header_raw($field);
+#pod   my @all_values  = $header->header_raw($field);
+#pod
+#pod This method returns the value or values of the given header field.  If the
+#pod named field does not appear in the header, this method returns false.
+#pod
+#pod =method header
+#pod
+#pod This method just calls C<header_raw>.  It's the older name for C<header_raw>,
+#pod but it can be a problem because L<Email::MIME>, a subclass of Email::Simple,
+#pod makes C<header> return the header's decoded value.
+#pod
+#pod =cut
 
-  my $first_value = $header->header($field);
-  my @all_values  = $header->header($field);
+sub _str_value { return ref $_[0] ? $_[0][0] : $_[0] }
 
-This method returns the value or values of the given header field.  If the
-named field does not appear in the header, this method returns false.
-
-=cut
-
-sub header {
+sub header_raw {
   my ($self, $field) = @_;
 
   my $headers  = $self->{headers};
   my $lc_field = lc $field;
 
   if (wantarray) {
-    return map { @$headers[ $_ * 2 + 1 ] }
+    return map { _str_value($headers->[ $_ * 2 + 1 ]) }
       grep { lc $headers->[ $_ * 2 ] eq $lc_field } 0 .. int($#$headers / 2);
   } else {
     for (0 .. int($#$headers / 2)) {
-      return $headers->[ $_ * 2 + 1 ] if lc $headers->[ $_ * 2 ] eq $lc_field;
+      return _str_value($headers->[ $_ * 2 + 1 ]) if lc $headers->[ $_ * 2 ] eq $lc_field;
     }
     return;
   }
 }
 
-=head2 header_set
+*header = \&header_raw;
 
-  $header->header_set($field => @values);
-
-This method updates the value of the given header.  Existing headers have their
-values set in place.  Additional headers are added at the end.  If no values
-are given to set, the header will be removed from to the message entirely.
-
-=cut
+#pod =method header_raw_set
+#pod
+#pod   $header->header_raw_set($field => @values);
+#pod
+#pod This method updates the value of the given header.  Existing headers have their
+#pod values set in place.  Additional headers are added at the end.  If no values
+#pod are given to set, the header will be removed from to the message entirely.
+#pod
+#pod =method header_set
+#pod
+#pod L<header_set> is another name for L<header_raw_set>, which was the original
+#pod name for the method and which you'll see most often.  In general, though, it's
+#pod better to be explicit and use L<header_raw_set>.  (In Email::MIME,
+#pod L<header_str_set> exists for letting the library do the header encoding for
+#pod you.)
+#pod
+#pod =cut
 
 # Header fields are lines composed of a field name, followed by a colon (":"),
 # followed by a field body, and terminated by CRLF.  A field name MUST be
@@ -199,7 +236,7 @@ are given to set, the header will be removed from to the message entirely.
 # However, a field body may contain CRLF when used in header "folding" and
 # "unfolding" as described in section 2.2.3.
 
-sub header_set {
+sub header_raw_set {
   my ($self, $field, @data) = @_;
 
   # I hate this block. -- rjbs, 2006-10-06
@@ -235,15 +272,20 @@ sub header_set {
   return wantarray ? @data : $data[0];
 }
 
-=head2 crlf
+sub header_set {
+  my ($self, $field, @data) = @_;
+  $self->header_raw_set($field, @data);
+}
 
-This method returns the newline string used in the header.
-
-=cut
+#pod =method crlf
+#pod
+#pod This method returns the newline string used in the header.
+#pod
+#pod =cut
 
 sub crlf { $_[0]->{mycrlf} }
 
-# =head2 fold
+# =method fold
 # 
 #   my $folded = $header->fold($line, \%arg);
 # 
@@ -274,15 +316,21 @@ sub _fold {
 
   my $indent = $arg->{indent} || $self->_default_fold_indent;
 
+  return $self->__fold_objless($line, $limit, $indent, $self->crlf);
+}
+
+sub __fold_objless {
+  my ($self, $line, $limit, $indent, $crlf) = @_;
+
   # We know it will not contain any new lines at present
   my $folded = "";
   while ($line) {
     if ($line =~ s/^(.{0,$limit})(\s|\z)//) {
-      $folded .= $1 . $self->crlf;
+      $folded .= $1 . $crlf;
       $folded .= $indent if $line;
     } else {
       # Basically nothing we can do. :(
-      $folded .= $line . $self->crlf;
+      $folded .= $line . $crlf;
       last;
     }
   }
@@ -290,7 +338,7 @@ sub _fold {
   return $folded;
 }
 
-# =head2 default_fold_at
+# =method default_fold_at
 # 
 # This method (provided for subclassing) returns the default length at which to
 # try to fold header lines.  The default default is 78.
@@ -299,7 +347,7 @@ sub _fold {
 
 sub _default_fold_at { 78 }
 
-# =head2 default_fold_indent
+# =method default_fold_indent
 # 
 # This method (provided for subclassing) returns the default string used to
 # indent folded headers.  The default default is a single space.
@@ -308,23 +356,133 @@ sub _default_fold_at { 78 }
 
 sub _default_fold_indent { " " }
 
-=head1 PERL EMAIL PROJECT
+1;
 
-This module is maintained by the Perl Email Project
+__END__
 
-L<http://emailproject.perl.org/>
+=pod
+
+=encoding UTF-8
+
+=head1 NAME
+
+Email::Simple::Header - the header of an Email::Simple message
+
+=head1 VERSION
+
+version 2.208
+
+=head1 SYNOPSIS
+
+  my $email = Email::Simple->new($text);
+
+  my $header = $email->header_obj;
+  print $header->as_string;
+
+=head1 DESCRIPTION
+
+This method implements the headers of an Email::Simple object.  It is a very
+minimal interface, and is mostly for private consumption at the moment.
+
+=head1 METHODS
+
+=head2 new
+
+  my $header = Email::Simple::Header->new($head, \%arg);
+
+C<$head> is a string containing a valid email header, or a reference to such a
+string.  If a reference is passed in, don't expect that it won't be altered.
+
+Valid arguments are:
+
+  crlf - the header's newline; defaults to CRLF
+
+=head2 as_string
+
+  my $string = $header->as_string(\%arg);
+
+This returns a stringified version of the header.
+
+=head2 header_names
+
+This method returns a list of the unique header names found in this header, in
+no particular order.
+
+=head2 header_raw_pairs
+
+  my @pairs = $header->header_raw_pairs;
+  my $first_name  = $pairs[0];
+  my $first_value = $pairs[1];
+
+This method returns a list of all the field/value pairs in the header, in the
+order that they appear in the header.  (Remember: don't try assigning that to a
+hash.  Some fields may appear more than once!)
+
+=head2 header_pairs
+
+L<header_pairs> is another name for L<header_raw_pairs>, which was the original
+name for the method and which you'll see most often.  In general, though, it's
+better to be explicit and use L<header_raw_pairs>.  (In Email::MIME,
+L<header_str_pairs> exists for letting the library do the header decoding for
+you.)
+
+=head2 header_raw
+
+  my $first_value = $header->header_raw($field);
+  my @all_values  = $header->header_raw($field);
+
+This method returns the value or values of the given header field.  If the
+named field does not appear in the header, this method returns false.
+
+=head2 header
+
+This method just calls C<header_raw>.  It's the older name for C<header_raw>,
+but it can be a problem because L<Email::MIME>, a subclass of Email::Simple,
+makes C<header> return the header's decoded value.
+
+=head2 header_raw_set
+
+  $header->header_raw_set($field => @values);
+
+This method updates the value of the given header.  Existing headers have their
+values set in place.  Additional headers are added at the end.  If no values
+are given to set, the header will be removed from to the message entirely.
+
+=head2 header_set
+
+L<header_set> is another name for L<header_raw_set>, which was the original
+name for the method and which you'll see most often.  In general, though, it's
+better to be explicit and use L<header_raw_set>.  (In Email::MIME,
+L<header_str_set> exists for letting the library do the header encoding for
+you.)
+
+=head2 crlf
+
+This method returns the newline string used in the header.
+
+=head1 AUTHORS
+
+=over 4
+
+=item *
+
+Simon Cozens
+
+=item *
+
+Casey West
+
+=item *
+
+Ricardo SIGNES
+
+=back
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2006-2007 by Ricardo SIGNES
+This software is copyright (c) 2003 by Simon Cozens.
 
-Copyright 2004 by Casey West
-
-Copyright 2003 by Simon Cozens
-
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
 =cut
-
-1;
